package com.yaohun.gameranking.api;

import com.yaohun.gameranking.GameRanking;
import com.yaohun.gameranking.data.UserData;
import com.yaohun.gameranking.data.UserJson;
import com.yaohun.gameranking.manager.RankManager;
import com.yaohun.gameranking.manager.UserManager;
import jsonrank.JSONObject;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class RankingAPI {

    public static LinkedHashMap<String,Integer> getRankingData(String timeKey,String dataKey){
        RankManager rankManager = GameRanking.getRankManager();
        HashMap<String,Integer> integerHashMap = new HashMap<>();
        List<UserData> userDataList = rankManager.getUserDataList(timeKey);
        for (UserData userData : userDataList){
            int points = userData.getPoints(dataKey);
            integerHashMap.put(userData.getUserName(),points);
        }
        // 将linkedHashMap的条目转换为列表
        List<Map.Entry<String, Integer>> list = new ArrayList<>(integerHashMap.entrySet());
        // 按值进行降序排序
        list.sort((entry1, entry2) -> entry2.getValue().compareTo(entry1.getValue()));
        // 创建一个新的LinkedHashMap并将排序后的条目插入其中
        LinkedHashMap<String, Integer> sortedMap = new LinkedHashMap<>();
        for (Map.Entry<String, Integer> entry : list) {
            sortedMap.put(entry.getKey(), entry.getValue());
        }
        return sortedMap;
    }

    public static LinkedHashMap<String,Integer> getRankingWeeklyData(String dataKey){
        RankManager rankManager = GameRanking.getRankManager();
        HashMap<String,Integer> integerHashMap = new HashMap<>();
        LocalDate today = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        List<String> dateList = new ArrayList<>();
        for (int i = 0; i < 7; i++) {
            LocalDate date = today.minusDays(i);
            String formattedDate = date.format(formatter);
            dateList.add(formattedDate);
        }
        for (String timeKey : dateList) {
            List<UserData> userDataList = rankManager.getUserDataList(timeKey);
            for (UserData userData : userDataList) {
                String userName = userData.getUserName();
                int points = userData.getPoints(dataKey);
                if(integerHashMap.get(userName) == null){
                    integerHashMap.put(userName, points);
                } else {
                    int nowAmount = integerHashMap.get(userName);
                    integerHashMap.put(userName, (nowAmount + points));
                }
            }
        }
        // 将linkedHashMap的条目转换为列表
        List<Map.Entry<String, Integer>> list = new ArrayList<>(integerHashMap.entrySet());
        // 按值进行降序排序
        list.sort((entry1, entry2) -> entry2.getValue().compareTo(entry1.getValue()));
        // 创建一个新的LinkedHashMap并将排序后的条目插入其中
        LinkedHashMap<String, Integer> sortedMap = new LinkedHashMap<>();
        for (Map.Entry<String, Integer> entry : list) {
            sortedMap.put(entry.getKey(), entry.getValue());
        }
        return sortedMap;
    }


    // 向今日某个数据增加数值
    public static void addUserValue(String userName,String valueKey,int points){
        RankManager manager = GameRanking.getRankManager();
        String timeKey = manager.getTime("yyyy-MM-dd");
        if(manager.isUserDataExit(timeKey,userName)){
            UserData userData = manager.getUserData(timeKey,userName);
            userData.addPoints(valueKey,points);
        } else {
            List<UserData> userDataList = manager.getUserDataList(timeKey);
            UserData userData = new UserData(userName);
            userDataList.add(userData);
            userData.addPoints(valueKey,points);
        }
    }

    // 导入观众数据
    public static void importAudienceData(String dataObjectString, String userName){
        UserManager manager = GameRanking.getUserManager();
        if(GameRanking.getUserManager().isUserJsonDataExit(userName)){
            UserJson userJson = manager.getUserJsonData(userName);
            JSONObject dataObject = new JSONObject(dataObjectString);
            userJson.setDataObject(dataObject);
        } else {
            manager.setUserJsonData(userName,new UserJson(dataObjectString));
        }
    }
}
